local M = {}

-- Use these along with the debug output on the console or the scenarioflashmessage debug to fine tune the failure points & other scenario features--
local brakingSumTrigger = 7500 -- The player can hold down the brakes as long as they want until we exceed this much cumulative braking
local brakingSumTrigger2 = 10500 -- After this much cumulative braking the brakes will fail the next time they are activated, no matter how little braking is done.
local failureHoldTime = 1.5 -- This is the amount of time the player must hold the brakes to trigger a failure (unless we exceed brakingSumTrigger2!)  
							-- Typically if you set 1.0 here the failure will occur after a total of 1.25s to 1.5s of braking depending on player timing.
local brakeReductionMultiplier = 0.25 -- We'll multiply all of the brakes' torque by this number at WP2 - the reduction makes the scenario much more dramatic!  
									  -- It also makes the other techniques more effective I think.
local hookupPenaltyThreshold = 30000 -- This is the limit after which the player will recieve a penalty for braking too much before WP2.  Remember that this is with full-power brakes.
local hookupPenalty = 2500 -- This is the actual penalty - remember to think of this in relation to brakingSumTrigger.
									  
M.brakingTorque = 0
M.parkingTorque = 0
M.brakeChecker = 0

local totalBrakingTorque = 0
local failureEnabled = 0
local brakingTime = 0
local brakingSum = 0
local brakingSumDisplayNotch = 5

local player = nil
local playerTrailer = nil

local function failBrakes()
	playerTrailer:queueLuaCommand('for a,b in pairs(drivetrain.wheels) do drivetrain.wheels[a].brakeCoreTemperature = 10000 drivetrain.wheels[a].brakeSurfaceTemperature = 10000 end')
	player:queueLuaCommand('for a,b in pairs(drivetrain.wheels) do drivetrain.wheels[a].brakeCoreTemperature = 10000 drivetrain.wheels[a].brakeSurfaceTemperature = 10000 end')
	player:queueLuaCommand('sounds.playSoundOnceAtNode("TurboBovSound", 224, 1); sounds.playSoundOnceAtNode("TireBurstSound", 83, 0.8)')
	failureEnabled = 0
	print ("Brakes failed!")
end

local function modifyBrakes(object, multiplier, target)
	-- If multiplier is > 0 we use it as a divisor against the target.  If it is equal to 0 then we zero out the target.
	if multiplier == 0 then
		object:queueLuaCommand('for a,b in pairs(drivetrain.wheels) do drivetrain.wheels[a].' .. target .. ' = 0 end')
	else
		object:queueLuaCommand('for a,b in pairs(drivetrain.wheels) do drivetrain.wheels[a].' .. target .. ' = drivetrain.wheels[a].' .. target .. ' * ' .. multiplier .. ' end')
	end
end

local function brakeFix()  -- it's a pun.  Get it?  It's also a big hack.  :-(

	-- for now we check to see if we previously reduced the brakes because we don't have a good way to 'reset' them
	-- if that's the case then we multiply by the inverse to bring them back up...
	-- The way we check is in the onRaceStart function - we queue commands to add up all wheels and then queue a command to run this function
	
	print ("Total Brake Torque for all wheels: " .. M.brakeChecker)
	
	if M.brakeChecker == 0 or M.brakeChecker == 40000 then
		guihooks.trigger('ScenarioFlashMessage', {{"Error in brakeFix() function!", 2}} )
	end
	
	if M.brakeChecker < 40000 then
		modifyBrakes(player, 1/brakeReductionMultiplier, "brakeTorque")
		modifyBrakes(playerTrailer, 1/brakeReductionMultiplier, "brakeTorque")
		print ("Fixed the brakes:")
		player:queueLuaCommand('for a,b in pairs(drivetrain.wheels) do print(drivetrain.wheels[a].brakeTorque) end')
		playerTrailer:queueLuaCommand('for a,b in pairs(drivetrain.wheels) do print(drivetrain.wheels[a].brakeTorque) end')
	end
end

local function setup()
    player = scenetree.findObject('scenario_player0')
	playerTrailer = scenetree.findObject('trailer')

	playerTrailer:queueLuaCommand("electrics.values.feet = 0")

	player:queueLuaCommand("for a,b in pairs(drivetrain.wheels) do obj:queueGameEngineLua('heavy_cargo_v2.brakeChecker = (heavy_cargo_v2.brakeChecker + '..drivetrain.wheels[a].brakeTorque..')') end")	
	playerTrailer:queueLuaCommand("for a,b in pairs(drivetrain.wheels) do obj:queueGameEngineLua('heavy_cargo_v2.brakeChecker = (heavy_cargo_v2.brakeChecker + '..drivetrain.wheels[a].brakeTorque..')') end")	
end

local function onRaceStart()

	heavy_cargo_v2.brakeFix()
		
	-- For now we just kill all parking brakes since I can't convince getWheel() to give me the parking brake torque.
	modifyBrakes(player, 0, "parkingTorque")
	modifyBrakes(playerTrailer, 0, "parkingTorque")

end

local function onRaceTick(raceTickTime)  
	player:queueLuaCommand("obj:queueGameEngineLua('heavy_cargo_v2.brakingTorque = ('..obj:getWheel(1).brakingTorque..')')")
	--  apparently get.Wheel().parkingTorque isn't a thing. ;-)
	--player:queueLuaCommand("obj:queueGameEngineLua('heavy_cargo_v2.parkingTorque = ('..obj:getWheel(1).parkingTorque..')')")
	
	totalBrakingTorque = M.brakingTorque + M.parkingTorque

	if totalBrakingTorque > 500 then
		brakingTime = brakingTime + raceTickTime
		brakingSum = brakingSum + (raceTickTime * totalBrakingTorque)
		print (brakingSumDisplayNotch .. " - Braking Sum: " .. brakingSum .. " - Braking: " .. M.brakingTorque .. " - Parking: " .. M.parkingTorque .. " - Braking Time: " .. brakingTime)
		if (brakingSum / 1000) > brakingSumDisplayNotch then 
			brakingSumDisplayNotch = 5 + (5 * math.floor(brakingSum / 5000))
			--guihooks.trigger('ScenarioFlashMessage', {{"Braking Sum: " .. math.floor(brakingSum), 2}} )
		end
	else
		brakingTime = 0
	end
	
	if failureEnabled == 1 then
		if brakingTime > failureHoldTime and brakingSum > brakingSumTrigger then
			print ("(normal brakingTime trigger - eg the breakingSumTrigger)")
			failBrakes()
		else
			if brakingSum > brakingSumTrigger2 then
				print ("(backup brakingSum trigger - eg the brakingSumTrigger2)")
				failBrakes()
			end
		end
	end
end

local function onRaceWaypoint(data, goal)
   
    if data.waypointName == 'WayPoint2' then
        if brakingSum > hookupPenaltyThreshold then 
			brakingSum = hookupPenalty
			print ("Resetting brakingSum to zero + " .. hookupPenalty .. " (penalty for hookup taking too much braking!).")
		else
			brakingSum = 0
			print ("Resetting brakingSum to zero (no penalty since hookup didn't take too much braking).")
		end
		
		failureEnabled = 1
		print ("Ready for brakes to fail...")
		
		modifyBrakes(player, brakeReductionMultiplier, "brakeTorque")
		modifyBrakes(playerTrailer, brakeReductionMultiplier, "brakeTorque")
    end

-- this (WP 0) seems to indicate that the scenario is fully loaded and we're at the scenario start dialog	
--	if data.cur == 0 then
--		setup()
--	end	   

end

M.onScenarioRestarted = setup
M.onScenarioLoaded = setup
M.onRaceTick = onRaceTick
M.onRaceWaypoint = onRaceWaypoint
M.onRaceStart = onRaceStart
M.brakeFix = brakeFix
M.onScenarioChange = onScenarioChange

return M